#!/usr/bin/python

import os
import sys
import time
import fcntl
import select
import signal
import socket
import optparse

def parseOpts():
    usage = "usage: %prog [options] url hostname"
    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-t", "--timeout", type="int", default=30, dest="timeout",
        help="Per-command timeout value in seconds (defaults to 30)")
    parser.add_option("-p", "--proxy", dest="proxy", help="Location of X509 " \
        "proxy certificate.")
    parser.add_option("-u", "--cert", dest="usercert", help="Location of " \
        "X509 certificate.")
    parser.add_option("-k", "--key", dest="userkey", help="Location of " \
        "X509 key.")
    opts, args = parser.parse_args()

    if opts.timeout < 0:
        raise ValueError("Timeout value cannot be negative")
    signal.alarm(opts.timeout+1)

    if len(args) != 2:
        parser.print_usage()
        print "URL and hostname must be specified"
        sys.exit(3)

    return opts, args

def run_timed_xrd(args, opts):
    sys.stdin.close()
    r, w = os.pipe()
    pid = os.fork()
    fl = fcntl.fcntl(r, fcntl.F_GETFL)
    try:
        fcntl.fcntl(r, fcntl.F_SETFL, fl | os.O_NDELAY)
    except AttributeError:
        fcntl.fcntl(r, fcntl.F_SETFL, fl | os.FNDELAY)
    if pid == 0:
        os.dup2(w, 1)
        os.dup2(w, 2)
        signal.alarm(opts.timeout)
        if opts.proxy:
            os.environ['XrdSecGSIUSERPROXY'] = opts.proxy
        if opts.usercert:
            os.environ['XrdSecGSIUSERCERT'] = opts.usercert
        if opts.userkey:
            os.environ['XrdSecGSIUSERKEY'] = opts.userkey
        os.execvp(args[0], args)
    output = ""
    while True:
        ready, _, _ = select.select([r], [], [], .5)
        if r in ready:
            output += os.read(r, 1024)
        pid, result = os.waitpid(pid, os.WNOHANG)
        if (pid, result) != (0, 0):
            break
    ready, _, _ = select.select([r], [], [], .01)
    if r in ready:
        output += os.read(r, 4096)

    if result:
        if os.WIFEXITED(result):
            print "Failed process exited %i (%s)." % (os.WEXITSTATUS(result),
                " ".join(args))
            sys.exit(2)
        elif os.WIFSIGNALED(result):
            if os.WTERMSIG(result) == signal.SIGALRM:
                print "Command %s did not exit within timeout %i." % ( \
                    " ".join(args), opts.timeout)
            else:
                print "Command %s failed due to signal %i." % (" ".join(args),
                    os.WTERMSIG(result))
            sys.exit(1)
        else:
            print "Read process exited due to unknown reason."
            sys.exit(3)
    return output

def main():
    opts, args = parseOpts()
    url, host = args
    cmd = ["cmsd_small", url, host]
    output = run_timed_xrd(cmd, opts)
    try:
        ips = socket.gethostbyaddr(host)[-1]
    except Exception, e:
        print e[-1]
        sys.exit(2)
    found_server = False
    for line in output.splitlines():
        info = line.split()
        if len(info) != 2:
            continue
        info = info[-1].split(":")
        if len(info) != 2:
            continue
        ip, port = info
        if ip in ips:
            found_server = True
    if found_server:
        print "CMSD server at %s appears OK." % host
    else:
        print "CMSD server at %s does not appear to respond." % host
        sys.exit(2)

if __name__ == '__main__':
    main()

